import { DecryptionFunction, EncryptionFunction, EncryptionType, IVGenerationFunction, KeyDerivationFunction, SaltGenerationFunction } from "./constructs";
interface ConfigurationOptions {
    /**
     * AES-CBC decryption function
     */
    decryption_cbc: DecryptionFunction;
    /**
     * AES-GCM decryption function
     */
    decryption_gcm: DecryptionFunction;
    /**
     * Default number of key derivation iterations
     */
    derivationRounds: number;
    /**
     * Keys derivation function
     */
    deriveKey: KeyDerivationFunction;
    /**
     * AES-CBC encryption function
     */
    encryption_cbc: EncryptionFunction;
    /**
     * AES-GCM encryption function
     */
    encryption_gcm: EncryptionFunction;
    /**
     * Random IV generation function
     */
    generateIV: IVGenerationFunction;
    /**
     * Random salt generation function
     */
    generateSalt: SaltGenerationFunction;
    /**
     * The encryption method - cbc/gcm
     */
    method: EncryptionType;
    /**
     * Salt character length
     */
    saltLength: number;
}
/**
 * Get the default options
 * @returns Default configuration options
 */
declare function getDefaultOptions(): ConfigurationOptions;
/**
 * System configuration
 */
export declare class Configuration {
    static getDefaultOptions: typeof getDefaultOptions;
    _options: ConfigurationOptions;
    /**
     * Configuration options
     * @memberof Configuration
     * @readonly
     */
    get options(): ConfigurationOptions;
    /**
     * Override the decryption method
     * @param method Which encryption type to override (cbc/gcm)
     * @param func A decryption function that should resemble that in the example
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.overrideDecryption("cbc", (encryptedComponents, keyDerivationInfo) => {
     *    // handle decryption
     *    // return Promise
     *  });
     */
    overrideDecryption(method: EncryptionType, func?: DecryptionFunction): Configuration;
    /**
     * Override the encryption method
     * @param method Which encryption type to override (cbc/gcm)
     * @param func A encryption function that should resemble that in the example
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.overrideEncryption("cbc", (text, keyDerivationInfo, ivBuffer) => {
     *    // handle encryption
     *    // return Promise
     *  });
     */
    overrideEncryption(method: EncryptionType, func?: EncryptionFunction): Configuration;
    /**
     * Override the IV generator
     * @param func The override function
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.overrideIVGeneration(() => {
     *    return Promise.resolve(ivBuffer);
     *  });
     */
    overrideIVGeneration(func?: IVGenerationFunction): Configuration;
    /**
     * Override key derivation
     * Derive the key according to the `pbkdf2` function in derivation.js
     * @param func The new key derivation function
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.overrideKeyDerivation((password, salt, rounds, bits) => {
     *    return Promise.resolve(derivedKeyBuffer);
     *  });
     */
    overrideKeyDerivation(func?: KeyDerivationFunction): Configuration;
    /**
     * Override salt generation
     * @param func The function to use for salt generation
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.overrideSaltGeneration(length => {
     *    return Promise.resolve(saltText);
     *  });
     */
    overrideSaltGeneration(func?: SaltGenerationFunction): Configuration;
    /**
     * Reset the configuration options
     * @memberof Configuration
     * @returns Returns self
     */
    reset(): Configuration;
    /**
     * Set the derivation rounds to use
     * @param rounds The new rounds to use (empty for reset)
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.setDerivationRounds(250000);
     */
    setDerivationRounds(rounds?: number): Configuration;
    /**
     * Set the encryption method to use
     * @param method The method to use (cbc/gcm)
     * @returns Returns self
     * @memberof Configuration
     * @example
     *  config.use("gcm");
     */
    use(method: EncryptionType): Configuration;
}
export {};
